/****************************************************************************************************
* Titolo:       BluetoothIOControl                                                                  *
* Descrizione:  Software gestione shield I/O MCP23017 tramite shield Bluetooth RN-42                *
* Autori:       Ingg. Tommaso Giusto e Alessandro Giusto                                            *
* Data:         20/09/2013                                                                          *
* Email:        tommro@libero.it                                                                    *
****************************************************************************************************/

/****************************************************************************************************
* Inclusione librerie                                                                               *
****************************************************************************************************/
#include <BluetoothRN42.h>
#include <Wire.h>
#include <IOExpanderMCP23017.h>
#include <EEPROM.h>

/****************************************************************************************************
* Definizione pin I/O Arduino                                                                       *
****************************************************************************************************/

// Led scheda
const int pinBoardLed = 13;
// Pin di interrupt input shield MCP23017
const int pinInterruptMCP23017 = 2;

/****************************************************************************************************
* Definizione struttura EEPROM                                                                      *
****************************************************************************************************/

/****************************************************************************************************
  0xFF,       // Byte ADD 000   Flag Reset EEPROM
                                  0x51 -> Non Eseguire Reset EEPROM
                                  Default -> Eseguire Reset EEPROM
  
  'B',        // Byte ADD 001   Nome Bluetooth del Sistema
  'T',        // Byte ADD 002   Default: 'BTIOControl' + 0xFF....
  'I',        // Byte ADD 003   Note: Max: 19 caratteri. Quelli non usati pari a 0xFF
  'O',        // Byte ADD 004
  'C',        // Byte ADD 005
  'o',        // Byte ADD 006
  'n',        // Byte ADD 007
  't',        // Byte ADD 008
  'r',        // Byte ADD 009
  'o',        // Byte ADD 00A
  'l',        // Byte ADD 00B
  0xFF,       // Byte ADD 00C
  0xFF,       // Byte ADD 00D
  0xFF,       // Byte ADD 00E
  0xFF,       // Byte ADD 00F
  0xFF,       // Byte ADD 010
  0xFF,       // Byte ADD 011
  0xFF,       // Byte ADD 012
  0xFF,       // Byte ADD 013
  
  '1',        // Byte ADD 014   PIN Code Bluetooth del Sistema
  '2',        // Byte ADD 015   Default: '1234' + 0xFF....
  '3',        // Byte ADD 016   Note: Max: 16 caratteri. Quelli non usati pari a 0xFF
  '4',        // Byte ADD 017
  0xFF,       // Byte ADD 018
  0xFF,       // Byte ADD 019
  0xFF,       // Byte ADD 01A
  0xFF,       // Byte ADD 01B
  0xFF,       // Byte ADD 01C
  0xFF,       // Byte ADD 01D
  0xFF,       // Byte ADD 01E
  0xFF,       // Byte ADD 01F
  0xFF,       // Byte ADD 020
  0xFF,       // Byte ADD 021
  0xFF,       // Byte ADD 022
  0xFF        // Byte ADD 023
****************************************************************************************************/
#define Flag_Reset_EEPROM_EEPROMADD         0x0000  // Flag Reset EEPROM

#define Start_Nome_Sistema_EEPROMADD        0x0001  // Struttura Nome Bluetooth del Sistema
#define Dimension_Nome_Sistema_EEPROM       19      // Byte struttura Nome Bluetooth del Sistema

#define Start_PIN_Code_Sistema_EEPROMADD    0x0014  // Struttura PIN Code Bluetooth del Sistema
#define Dimension_PIN_Code_Sistema_EEPROM   16      // Byte struttura PIN Code Bluetooth del Sistema

/****************************************************************************************************
* Definizione protocollo di comunicazione bluetootin I/O Arduino                                    *
****************************************************************************************************/

// PROTOCOLLO RICHIESTE:
// Byte 0 -> Codice comando
//             GETNFO -> lettura
//             SETNFO -> scrittura
// Byte 1 -> Identificativo risorsa
//             STATUS -> pagina stato scheda (GETNFO)
//             OUTRLE -> uscite rele (GETNFO e SETNFO)
//             INPDIG -> ingressi digitali (GETNFO)
// Byte 2 -> Numero IO Expander (GETNFO e SETNFO)
// Byte 3 -> Numero risorsa (GETNFO e SETNFO)
// Byte 4 -> Stato risorsa (SETNFO)

// PROTOCOLLO RISPOSTE:
// Alla richiesta connessione -> WLCOME
// Al comando: GETNFO STATUS -> Stato I/O scheda
//                                  Byte 0 -> Stato shield IO Expander rilevati
//                                  Byte 1 -> Stato uscite rele IO Expander 0
//                                  Byte 2 -> Stato ingressi digitali IO Expander 0
//                                  Byte 3 -> Stato uscite rele IO Expander 1
//                                  Byte 4 -> Stato ingressi digitali IO Expander 1
//                                  Byte 5 -> Stato uscite rele IO Expander 2
//                                  Byte 6 -> Stato ingressi digitali IO Expander 2
//                                  Byte 7 -> Stato uscite rele IO Expander 3
//                                  Byte 8 -> Stato ingressi digitali IO Expander 3
//                                  Byte 9 -> Stato uscite rele IO Expander 4
//                                  Byte 10 -> Stato ingressi digitali IO Expander 4
//                                  Byte 11 -> Stato uscite rele IO Expander 5
//                                  Byte 12 -> Stato ingressi digitali IO Expander 5
//                                  Byte 13 -> Stato uscite rele IO Expander 6
//                                  Byte 14 -> Stato ingressi digitali IO Expander 6
//                                  Byte 15 -> Stato uscite rele IO Expander 7
//                                  Byte 16 -> Stato ingressi digitali IO Expander 7
// Al comando: SETNFO -> Risposta positiva OKSTNF

// Risposta richiesta connessione:(WLCOME)
#define WLCOME_CMD      0x40

// Comando richiesta lettura dati (GET)
#define GETNFO_CMD      0x41

// Comando richiesta scrittura dati (SET)
#define SETNFO_CMD      0x42

// Risposta positiva comando SETNFO: (OKSTNF)
#define OKSTNF_CMD      0x43

// Risposta negativa comando SETNFO: (ERSTNF)
#define ERSTNF_CMD      0x44

// Risposta negativa comando GETNFO: (ERGTNF)
#define ERGTNF_CMD      0x45

// Comando identificativo pagina stato scheda (GET STATUS)
#define STATUS_CMD      0x40

// Comando identificativo uscite rele (GET OUTRLE_CMD e SET OUTRLE_CMD)
#define OUTRLE_CMD      0x41

// Comando identificativo ingressi digitali fotoaccoppiati (GET INPDIG_CMD)
#define INPDIG_CMD      0x42

// Comando identificativo nome bluetooth del sistema (SET NMEBTH)
#define NMEBTH_CMD      0x43

// Comando identificativo pin code bluetooth del sistema (SET PNCBT)
#define PNCBTH_CMD      0x44

/****************************************************************************************************
* Definizione I/O shield MCP23017
****************************************************************************************************/

#define INPUT1_PIN 0
#define INPUT2_PIN 1
#define INPUT3_PIN 2
#define INPUT4_PIN 3
#define INPUT5_PIN 4
#define INPUT6_PIN 5
#define INPUT7_PIN 6
#define INPUT8_PIN 7

#define OUTPUT1_PIN 15
#define OUTPUT2_PIN 14
#define OUTPUT3_PIN 13
#define OUTPUT4_PIN 12
#define OUTPUT5_PIN 11
#define OUTPUT6_PIN 10
#define OUTPUT7_PIN 9
#define OUTPUT8_PIN 8

/****************************************************************************************************
* Definizione Costanti                                                                              *
****************************************************************************************************/

// Dimensione massima (in byte) del buffer di trasmissione/ricezione bluetooth
#define Num_Max_Byte_Buffer_Bluetooth           35

/****************************************************************************************************
* Variabili globali                                                                                 *
****************************************************************************************************/

// Oggetto shield bluetooth RN-42
BLUETOOTHRN42 bluetooth;

// Oggetto shield I/O MCP23017
IOExpanderMCP23017 IOExpander[8]; 

// Flag IO Expander rilevati
//  Bit = 0 -> IO Expander non rilevato
//  Bit = 1 -> IO Expander rilevato
char IOExpanderRilevati = 0x00;

// Stato attuale uscite relè (tutti IO Expander rilevati)
char Stato_Uscite_Rele[8];
// Stato attuale ingressi digitali (tutti IO Expander rilevati)
char Stato_Ingressi_Digitali[8];

/****************************************************************************************************
* Codice programma                                                                                  *
****************************************************************************************************/

// Inizializzazione Scheda
void setup() {
  char i;
  
  // Resetto stato I/O
  memset(Stato_Ingressi_Digitali, 0x00, sizeof(Stato_Ingressi_Digitali));
  memset(Stato_Uscite_Rele, 0x00, sizeof(Stato_Uscite_Rele));

  // Inizializzo I/O
  pinMode(pinBoardLed, OUTPUT);
  digitalWrite(pinBoardLed, LOW);
  pinMode(pinInterruptMCP23017, INPUT);       
  // Attesa accensione
  delay(1500);

  // Se eseguire reset EEPROM
  if (EEPROM.read(Flag_Reset_EEPROM_EEPROMADD) != 0x51) {
      // Reset EEPROM
      // Nome Bluetooth del Sistema
      EEPROM.write(Start_Nome_Sistema_EEPROMADD, 'B');
      EEPROM.write(Start_Nome_Sistema_EEPROMADD + 1, 'T');
      EEPROM.write(Start_Nome_Sistema_EEPROMADD + 2, 'I');
      EEPROM.write(Start_Nome_Sistema_EEPROMADD + 3, 'O');
      EEPROM.write(Start_Nome_Sistema_EEPROMADD + 4, 'C');
      EEPROM.write(Start_Nome_Sistema_EEPROMADD + 5, 'o');
      EEPROM.write(Start_Nome_Sistema_EEPROMADD + 6, 'n');
      EEPROM.write(Start_Nome_Sistema_EEPROMADD + 7, 't');
      EEPROM.write(Start_Nome_Sistema_EEPROMADD + 8, 'r');
      EEPROM.write(Start_Nome_Sistema_EEPROMADD + 9, 'o');
      EEPROM.write(Start_Nome_Sistema_EEPROMADD + 10, 'l');
      EEPROM.write(Start_Nome_Sistema_EEPROMADD + 11, 0xFF);
      EEPROM.write(Start_Nome_Sistema_EEPROMADD + 12, 0xFF);
      EEPROM.write(Start_Nome_Sistema_EEPROMADD + 13, 0xFF);
      EEPROM.write(Start_Nome_Sistema_EEPROMADD + 14, 0xFF);
      EEPROM.write(Start_Nome_Sistema_EEPROMADD + 15, 0xFF);
      EEPROM.write(Start_Nome_Sistema_EEPROMADD + 16, 0xFF);
      EEPROM.write(Start_Nome_Sistema_EEPROMADD + 17, 0xFF);
      EEPROM.write(Start_Nome_Sistema_EEPROMADD + 18, 0xFF);
      // PIN Code Bluetooth del Sistema
      EEPROM.write(Start_PIN_Code_Sistema_EEPROMADD, '1');
      EEPROM.write(Start_PIN_Code_Sistema_EEPROMADD + 1, '2');
      EEPROM.write(Start_PIN_Code_Sistema_EEPROMADD + 2, '3');
      EEPROM.write(Start_PIN_Code_Sistema_EEPROMADD + 3, '4');
      EEPROM.write(Start_PIN_Code_Sistema_EEPROMADD + 4, 0xFF);
      EEPROM.write(Start_PIN_Code_Sistema_EEPROMADD + 5, 0xFF);
      EEPROM.write(Start_PIN_Code_Sistema_EEPROMADD + 6, 0xFF);
      EEPROM.write(Start_PIN_Code_Sistema_EEPROMADD + 7, 0xFF);
      EEPROM.write(Start_PIN_Code_Sistema_EEPROMADD + 8, 0xFF);
      EEPROM.write(Start_PIN_Code_Sistema_EEPROMADD + 9, 0xFF);
      EEPROM.write(Start_PIN_Code_Sistema_EEPROMADD + 10, 0xFF);
      EEPROM.write(Start_PIN_Code_Sistema_EEPROMADD + 11, 0xFF);
      EEPROM.write(Start_PIN_Code_Sistema_EEPROMADD + 12, 0xFF);
      EEPROM.write(Start_PIN_Code_Sistema_EEPROMADD + 13, 0xFF);
      EEPROM.write(Start_PIN_Code_Sistema_EEPROMADD + 14, 0xFF);
      EEPROM.write(Start_PIN_Code_Sistema_EEPROMADD + 15, 0xFF);
      // Flag Reset EEPROM
      EEPROM.write(Flag_Reset_EEPROM_EEPROMADD, 0x51);
  }    // Se eseguire reset EEPROM

  // Avvio librerie Wire
  Wire.begin();
  
  // Avvio/rilevo libreria MCP23017
  IOExpanderRilevati = 0x00;  
  for (i = 0; i < 8; i++) {
    // Avvio libreria MCP23017
    IOExpander[i].begin(i);

    // Se rilevo MCP23017
    if (IOExpander[i].init() == true) {
      // Setto flag IO Expander rilevato
      setIOExpanderRilevato(i);
      // Inizializzo modalità pin I/O MCP23017
      IOExpander[i].pinMode(INPUT1_PIN,  INPUT);
      IOExpander[i].pinMode(INPUT2_PIN,  INPUT);
      IOExpander[i].pinMode(INPUT3_PIN,  INPUT);
      IOExpander[i].pinMode(INPUT4_PIN,  INPUT);
      IOExpander[i].pinMode(INPUT5_PIN,  INPUT);
      IOExpander[i].pinMode(INPUT6_PIN,  INPUT);
      IOExpander[i].pinMode(INPUT7_PIN,  INPUT);
      IOExpander[i].pinMode(INPUT8_PIN,  INPUT);
      IOExpander[i].pinMode(OUTPUT1_PIN,  OUTPUT);
      IOExpander[i].pinMode(OUTPUT2_PIN,  OUTPUT);
      IOExpander[i].pinMode(OUTPUT3_PIN,  OUTPUT);
      IOExpander[i].pinMode(OUTPUT4_PIN,  OUTPUT);
      IOExpander[i].pinMode(OUTPUT5_PIN,  OUTPUT);
      IOExpander[i].pinMode(OUTPUT6_PIN,  OUTPUT);
      IOExpander[i].pinMode(OUTPUT7_PIN,  OUTPUT);
      IOExpander[i].pinMode(OUTPUT8_PIN,  OUTPUT);
      
      // Inizializzo interrupt pin input MCP23017
      IOExpander[i].pinEnableINT(INPUT1_PIN);
      IOExpander[i].pinEnableINT(INPUT2_PIN);
      IOExpander[i].pinEnableINT(INPUT3_PIN);
      IOExpander[i].pinEnableINT(INPUT4_PIN);  
      IOExpander[i].pinEnableINT(INPUT5_PIN);
      IOExpander[i].pinEnableINT(INPUT6_PIN);
      IOExpander[i].pinEnableINT(INPUT7_PIN);
      IOExpander[i].pinEnableINT(INPUT8_PIN);      
    }    // Chiusura if rilevo MCP23017
  }    // Chiusura ciclo for avvio/rilevo libreria MCP23017
}    // Chiusura funzione setup

// Programma Principale
void loop() {
  // Variabili buffer bluetooth inRX/TX
  char Buffer_Bluetooth_RX[Num_Max_Byte_Buffer_Bluetooth];
  char Buffer_Bluetooth_TX[Num_Max_Byte_Buffer_Bluetooth];
  // Flag connessione bluetooth attiva
  bool ConnessioneAttiva = false;
  // Flag eseguire config bluetooth
  bool EseguireConfigBluetooth = true;
  int Num_Byte;
  int timeOut;  
  int i, j;

  // Ciclo infinito di esecuzione
  for (;;) {
    // Se rilevato interrupt input shield MCP23017
    if (digitalRead(pinInterruptMCP23017) == 0) {
      // Considero IO Expander rilevati
      for (j = 0; j < 8; j++) {
        // Se IO Expander rilevato
        if (IOExpanderRilevato(j) == true) {
          // Azzero stato ingressi digitali
          Stato_Ingressi_Digitali[j] = 0x00;
          // Leggo stato ingressi digitali
          for (i = 0; i < 8; i++) {
            if (IOExpander[j].digitalRead(INPUT1_PIN + i) == HIGH)
              Stato_Ingressi_Digitali[j] |= (1 << i);
          }
        }    // Chiusura if IO Expander rilevato
      }    // Chiusura ciclo for considero IO Expander rilevati
    }    // Chiusura if rilevato interrupt input shield MCP23017

    // Se nessuna connessione attiva
    if (ConnessioneAttiva == false) {
      // Se eseguire config bluetooth
      if (EseguireConfigBluetooth == true) {
        // Configurazione bluetooth RN-42
        configBluetooth();
        // Reset flag reset modulo
        EseguireConfigBluetooth = false;
      }    // Chiusura if eseguire config bluetooth

      // Se ricevuti byte dal modulo
      if (bluetooth.Available_RN_42() != 0) {
        // Se richiesta nuova connessione
        if (bluetooth.New_Connection_Request_RN_42() == true) {
          // Attivo led segnalazione
          digitalWrite(pinBoardLed, HIGH);
          // Invio risposta richiesta connessione: WLCOME
          Buffer_Bluetooth_TX[0] = WLCOME_CMD;
          bluetooth.Write_Buffer_Bluetooth_RN_42(Buffer_Bluetooth_TX, Num_Max_Byte_Buffer_Bluetooth);
          
          // Avvio scansione time out
          timeOut = 0;

          // Indico connessione collegata
          ConnessioneAttiva = true;
        }     // Chiusura if richiesta nuova connessione
      }     // Chiusura if ricevuti byte dal modulo        
    }    // Chiusura if nessuna connessione attiva
    // Se connessione attiva
    else {
      // Se ricevuti byte dal modulo
      if (bluetooth.Available_RN_42() != 0) {
        // Se rilevata disconnessione
        if (bluetooth.Close_Connection_Request_RN_42() == true) {
          // Disattivo led segnalazione
          digitalWrite(pinBoardLed, LOW);
          // Indico connessione non collegata
          ConnessioneAttiva = false;
        }     // Chiusura if rilevata disconnessione

        // Se non rilevata disconnessione
        else {
          // Leggo i byte dal buffer di ricezione
          Num_Byte = bluetooth.Read_Buffer_Bluetooth_RN_42(Buffer_Bluetooth_RX);
          // Seleziono codice comando ricevuto
          switch (Buffer_Bluetooth_RX[0]) {
            // Se ricevuto comando richiesta lettura dati (GET)
            case GETNFO_CMD:
              // Seleziono identificativo risorsa
              switch (Buffer_Bluetooth_RX[1]) {
                // Se ricevuto identificativo pagina stato scheda (GET STATUS)
                case STATUS_CMD:
                  // Invio Stato shield IO Expander rilevati
                  Buffer_Bluetooth_TX[0] = IOExpanderRilevati;
                  // Invio Stato uscite rele/ingressi digitali IO Expander
                  for (j = 0; j < 8; j++) {
                    // Se IO Expander selezionato non rilevato
                    if (IOExpanderRilevato(j) == false) {
                      // Azzero stato I/O Expander
                      Stato_Uscite_Rele[j] = 0x00;
                      Stato_Ingressi_Digitali[j] = 0x00;
                    }

                    // Invio stato I/O Expander
                    Buffer_Bluetooth_TX[1 + (j * 2)] = Stato_Uscite_Rele[j];
                    Buffer_Bluetooth_TX[2 + (j * 2)] = Stato_Ingressi_Digitali[j];
                  }
                  bluetooth.Write_Buffer_Bluetooth_RN_42(Buffer_Bluetooth_TX, Num_Max_Byte_Buffer_Bluetooth);                                          
                  break;      // Chiusura case ricevuto identificativo pagina stato scheda (GET STATUS)

                // Se ricevuto identificativo uscite rele (GET OUTRLE_CMD)
                case OUTRLE_CMD:
                  // Se IO Expander selezionato rilevato                
                  if (IOExpanderRilevato(Buffer_Bluetooth_RX[2]) == true)
                    // Invio stato uscita rele selezionata
                    Buffer_Bluetooth_TX[0] = Stato_Uscite_Rele[Buffer_Bluetooth_RX[2]];
                  // Se IO Expander selezionato non rilevato
                  else
                    // Invio risposta negativa comando GETNFO: (ERGTNF)
                    Buffer_Bluetooth_TX[0] = ERGTNF_CMD;

                  bluetooth.Write_Buffer_Bluetooth_RN_42(Buffer_Bluetooth_TX, Num_Max_Byte_Buffer_Bluetooth);
                  break;      // Chiusura case ricevuto identificativo uscite rele (GET OUTRLE_CMD)

                // Se ricevuto identificativo ingressi digitali (GET INPDIG)
                case INPDIG_CMD:
                  // Se IO Expander selezionato rilevato                
                  if (IOExpanderRilevato(Buffer_Bluetooth_RX[2]) == true)
                    // Invio stato ingresso digitale selezionato
                    Buffer_Bluetooth_TX[0] = Stato_Ingressi_Digitali[Buffer_Bluetooth_RX[2]];
                  // Se IO Expander selezionato non rilevato
                  else
                    // Invio risposta negativa comando GETNFO: (ERGTNF)
                    Buffer_Bluetooth_TX[0] = ERGTNF_CMD;

                  bluetooth.Write_Buffer_Bluetooth_RN_42(Buffer_Bluetooth_TX, Num_Max_Byte_Buffer_Bluetooth);
                  break;      // Chiusura case ricevuto identificativo ingressi digitali (GET INPDIG)
              }     // Chiusura switch seleziono identificativo risorsa
              break;      // Chiusura case ricevuto comando richiesta lettura dati (GET)

              // Se ricevuto comando richiesta scrittura dati (SET)
              case SETNFO_CMD:
                // Seleziono identificativo risorsa
                switch (Buffer_Bluetooth_RX[1]) {
                  // Se ricevuto identificativo uscite rele (SET OUTRLE_CMD)
                  case OUTRLE_CMD:
                    // Se ricevuti parametri corretti
                    if ((Buffer_Bluetooth_RX[2] >= 0) && (Buffer_Bluetooth_RX[2] <= 7) &&                    
                        (Buffer_Bluetooth_RX[3] >= 0) && (Buffer_Bluetooth_RX[3] <= 7) &&
                        (Buffer_Bluetooth_RX[4] >= 0) && (Buffer_Bluetooth_RX[4] <= 1)) {
                      // Imposto stato uscita rele
                      IOExpander[Buffer_Bluetooth_RX[2]].digitalWrite(OUTPUT1_PIN - (int) (Buffer_Bluetooth_RX[3]), (int) (Buffer_Bluetooth_RX[4]));
                      // Memorizzo stato uscita rele attuale
                      if (Buffer_Bluetooth_RX[4] == 0)
                        Stato_Uscite_Rele[Buffer_Bluetooth_RX[2]] &= (~(1 << Buffer_Bluetooth_RX[3]));
                      else if (Buffer_Bluetooth_RX[4] == 1)
                        Stato_Uscite_Rele[Buffer_Bluetooth_RX[2]] |= (1 << Buffer_Bluetooth_RX[3]);

                      // Invio risposta positiva comando SETNFO: (OKSTNF)
                      Buffer_Bluetooth_TX[0] = OKSTNF_CMD;
                    }    // Chiusura if ricevuti parametri corretti
                    // Se ricevuti parametri errati
                    else
                      // Invio risposta negativa comando SETNFO: (ERSTNF)
                      Buffer_Bluetooth_TX[0] = ERSTNF_CMD;

                    bluetooth.Write_Buffer_Bluetooth_RN_42(Buffer_Bluetooth_TX, Num_Max_Byte_Buffer_Bluetooth);                      
                    break;    // Chiusura case ricevuto identificativo uscite rele (SET OUTRLE_CMD)
                  
                  // Se ricevuto identificativo nome bluetooth del sistema (SET NMEBTH_CMD)
                  case NMEBTH_CMD:
                    // Memorizzo in EEPROM carattere nome ricevuto
                    EEPROM.write(Start_Nome_Sistema_EEPROMADD + Buffer_Bluetooth_RX[2], Buffer_Bluetooth_RX[3]);

                    // Se ricevuto ultimo carattere nome
                    if (Buffer_Bluetooth_RX[2] == (Dimension_Nome_Sistema_EEPROM - 1))
                      // Set flag reset modulo
                      EseguireConfigBluetooth = true;
                    
                    // Invio risposta positiva comando SETNFO: (OKSTNF)
                    Buffer_Bluetooth_TX[0] = OKSTNF_CMD;
                    bluetooth.Write_Buffer_Bluetooth_RN_42(Buffer_Bluetooth_TX, Num_Max_Byte_Buffer_Bluetooth);
                    break;      // Chiusura case ricevuto identificativo nome bluetooth del sistema (SET NMEBTH_CMD)

                  // Se ricevuto identificativo pin code bluetooth del sistema (SET PNCBTH_CMD)
                  case PNCBTH_CMD:
                    // Memorizzo in EEPROM carattere PIN code ricevuto
                    EEPROM.write(Start_PIN_Code_Sistema_EEPROMADD + Buffer_Bluetooth_RX[2], Buffer_Bluetooth_RX[3]);

                    // Se ricevuto ultimo carattere PIN code
                    if (Buffer_Bluetooth_RX[2] == (Dimension_PIN_Code_Sistema_EEPROM - 1))
                      // Set flag reset modulo
                      EseguireConfigBluetooth = true;

                    // Invio risposta positiva comando SETNFO: (OKSTNF)
                    Buffer_Bluetooth_TX[0] = OKSTNF_CMD;
                    bluetooth.Write_Buffer_Bluetooth_RN_42(Buffer_Bluetooth_TX, Num_Max_Byte_Buffer_Bluetooth);
                    break;      // Chiusura case ricevuto identificativo pin code bluetooth del sistema (SET PNCBTH_CMD)                         

                 // Se ricevuto identificativo errato
                 default:
                   // Invio risposta negativa comando SETNFO: (ERSTNF)
                   Buffer_Bluetooth_TX[0] = ERSTNF_CMD;
                   bluetooth.Write_Buffer_Bluetooth_RN_42(Buffer_Bluetooth_TX, Num_Max_Byte_Buffer_Bluetooth);
                   break;      // Chiusura case ricevuto identificativo errato
               }     // Chiusura switch seleziono identificativo risorsa
               break;      // Chiusura case ricevuto comando richiesta scrittura dati (GET)
          }     // Chiusura switch seleziono codice comando ricevuto          
        }    // Chiusura else non rilevata disconnessione

        // Resetto scansione time out
        timeOut = 0;
      }     // Chiusura if ricevuti byte dal modulo

      // Se non ricevuti byte dal modulo
      else {
        // Attesa time out
        delay(250);
        // Incremento timeout connessione
        timeOut++;
        // Se time out connessione scaduto
        if ((timeOut * 250) >= 5000) {
          // Chiudo connessione attuale modulo RN-42 (reboot)
          bluetooth.Kill_Connection_RN_42();
          // Segnalo disconnessione
          for (i = 0; i < 10; i++) {
            digitalWrite(pinBoardLed, HIGH);
            delay(50);
            digitalWrite(pinBoardLed, LOW);
            delay(50);
          }    // Segnalo disconnessione

          // Indico connessione non collegata
          ConnessioneAttiva = false;
        }     // Chiusura if time out connessione scaduto
      }     //  Chiusura else non ricevuti byte dal modulo 
    }    // Chiusura else connessione attiva
  }    // Ciclo infinito di esecuzione
}    // Chiusura funzione loop

// Configurazione bluetooth RN-42
void configBluetooth() {
  String Nome = "";
  String PIN = "";  
  char NomePINChar;

  // Begin software for bluetooth
  bluetooth.Begin();
  // Enter in command mode
  bluetooth.Enter_Command_Mode_RN_42();
  // Set factory default
  bluetooth.Set_Factory_Default_RN_42();
  // Set bluetooth operation mode as slave (0)
  bluetooth.Set_Operation_Mode_Slave_RN_42();
  // Set Extended Status String RN-42 module
  bluetooth.Set_Extended_Status_String_RN_42();
  // Set config timer RN-42 module
  bluetooth.Set_Config_Timer_RN_42();
  // Set Inquiry Scan Window RN-42 module
  bluetooth.Set_Inquiry_Scan_Window_RN_42();
  // Set Page Scan Window RN-42 module
  bluetooth.Set_Page_Scan_Window_RN_42();

  // Leggo i caratteri nome bluetooth
  for (int tmp = 0; tmp < Dimension_Nome_Sistema_EEPROM; tmp++) {
    // Leggo singolo carattere
    NomePINChar = EEPROM.read(Start_Nome_Sistema_EEPROMADD + tmp);
    // Se trovato carattere fine nome
    if (NomePINChar == (char) (0xFF))
      // Blocco ciclo for leggo i caratteri nome bluetooth
      break;

    // Accodo carattere letto
    Nome = Nome + NomePINChar;
  }
  // Set name of bluetooth
  bluetooth.Set_Name_RN_42(Nome);
  
  // Leggo i caratteri PIN bluetooth
  for (int tmp = 0; tmp < Dimension_PIN_Code_Sistema_EEPROM; tmp++) {
    // Leggo singolo carattere
    NomePINChar = EEPROM.read(Start_PIN_Code_Sistema_EEPROMADD + tmp);
    // Se trovato carattere fine PIN
    if (NomePINChar == (char) (0xFF))
      // Blocco ciclo for leggo i caratteri PIN bluetooth
      break;

    // Accodo carattere letto
    PIN = PIN + NomePINChar;
  }
  // Set PIN of bluetooth
  bluetooth.Set_PIN_Code_RN_42(PIN);  

  // Enter in data mode
  bluetooth.Enter_Data_Mode_RN_42();
  // Reboot RN-42 module
  bluetooth.Reboot_RN_42();
}    // Chiusura funzione configBluetooth

// Verifica se un IO Expander è stato rilevato
boolean IOExpanderRilevato(int ExpanderNum) {
  // Se IO Expander selezionato non rilevato
  if ((IOExpanderRilevati & (1 << ExpanderNum)) == 0x00)
    return(false);
  // Se IO Expander selezionato rilevato
  else
    return(true);
}    // Chiusura funzione IOExpanderRilevato

// Setta flag IO Expander è stato rilevato
void setIOExpanderRilevato(int ExpanderNum) {
  // Setto flag IO Expander rilevato
  IOExpanderRilevati |= (1 << ExpanderNum);
}    // Chiusura funzione setIOExpanderRilevato

// Resetta flag IO Expander è stato rilevato
void resetIOExpanderRilevato(int ExpanderNum) {
  // Resetto flag IO Expander rilevato
  IOExpanderRilevati &= (~(1 << ExpanderNum));
}    // Chiusura funzione resetIOExpanderRilevato
